searchData={"content_type":"text/markdown","items":[{"doc":"ToxiproxyEx is an Elixir API client for the resilience testing tool Toxiproxy.\n\n[Toxiproxy](https://github.com/shopify/toxiproxy) is a proxy to simulate network\nand system conditions. The Elixir API aims to make it simple to write tests that\nensure your application behaves appropriately under harsh conditions. Before you\ncan use the Elixir library, you need to read the [Usage section of the Toxiproxy\nREADME](https://github.com/shopify/toxiproxy#usage).","ref":"ToxiproxyEx.html","title":"ToxiproxyEx","type":"module"},{"doc":"By default the Elixir client communicates with the Toxiproxy daemon via HTTP on `http://127.0.0.1:8474`, but you can point to any host via your application configuration:\n```elixir\nconfig :toxiproxy_ex, host: \"http://toxiproxy.local:8844\"\n```\n\nFor example, to simulate 1000ms latency on a database server you can use the\n`latency` toxic with the `latency` argument (see the Toxiproxy project for a\nlist of all toxics):\n\n```elixir\nToxiproxyEx.get!(:mysql_master)\n|> ToxiproxyEx.toxic(:latency, latency: 1000)\n|> ToxiproxyEx.apply!(fn ->\n  Repo.all(Shop) # this took at least 1s\nend)\n```\n\nYou can also take an endpoint down for the duration of a function at the TCP level:\n\n```elixir\nToxiproxyEx.get!(:mysql_master)\n|> ToxiproxyEx.down!(fn ->\n  Repo.all(Shop) # this'll raise\nend)\n```\n\nIf you want to simulate all your Redis instances being down:\n\n```elixir\nToxiproxyEx.grep!(~r/redis/)\n|> ToxiproxyEx.down!(fn ->\n  # any redis call will fail\nend)\n```\n\nIf you want to simulate that your cache server is slow at incoming network\n(upstream), but fast at outgoing (downstream), you can apply a toxic to just the\nupstream:\n\n```elixir\nToxiproxyEx.get!(:cache)\n|> ToxiproxyEx.upstream(:latency, latency: 1000)\n|> ToxiproxyEx.apply!(fn ->\n  Cache.get(:omg) # will take at least a second\nend)\n```\n\nBy default the toxic is applied to the downstream connection, you can be\nexplicit and compose them:\n\n```elixir\nToxiproxyEx.grep!(~r/redis/)\n|> ToxiproxyEx.upstream(:slow_close, delay: 100)\n|> ToxiproxyEx.downstream(:latency, jitter: 300)\n|> ToxiproxyEx.apply!(fn ->\n  # all redises are now slow at responding and closing\nend)\n```\n\nSee the [Toxiproxy README](https://github.com/shopify/toxiproxy#Toxics) for a\nlist of toxics.","ref":"ToxiproxyEx.html#module-usage","title":"Usage - ToxiproxyEx","type":"module"},{"doc":"To populate Toxiproxy pass the proxy configurations to `ToxiproxyEx.populate!`:\n\n```elixir\nToxiproxyEx.populate!([\n  %{\n    name: \"mysql_master\",\n    listen: \"localhost:21212\",\n    upstream: \"localhost:3306\",\n  },\n  %{\n    name: \"mysql_read_only\",\n    listen: \"localhost:21213\",\n    upstream: \"localhost:3306\",\n  }\n])\n```\n\nThis will create the proxies passed, or replace the proxies if they already exist in Toxiproxy.\nIt's recommended to do this as early in your application startup process as possible, see the\n[Toxiproxy README](https://github.com/shopify/toxiproxy#usage). If you have many\nproxies, we recommend storing the Toxiproxy configs in a configuration file and\ndeserializing it into `ToxiproxyEx.populate!/1`.","ref":"ToxiproxyEx.html#module-populate","title":"Populate - ToxiproxyEx","type":"module"},{"doc":"This library made the choice to use exceptions on the public API methods to signal errors.\n\nThis was chosen since this is a library meant to be used in testing code only, where you want test cases to fail if your set assumptions are not met. In this sense setting assumptions that will not be met (toxiproxy-server is not running, passing invalid configurations) is considered to be a developer error and should be fixed rather than handled in code.\n\n**Server Errors**\n\nIf any API interaction with toxiproxy fails, a `ServerError` will be raised.\n\n**Client Errors**\n\nIf you miss-configure toxiproxy via the elixir API, an `ArgumentError` will be raised.","ref":"ToxiproxyEx.html#module-error-handling","title":"Error Handling - ToxiproxyEx","type":"module"},{"doc":"Retrieves a list of all proxies from the toxiproxy server.\n\nRaises `ToxiproxyEx.ServerError` if the list of proxies could not be retrieved.","ref":"ToxiproxyEx.html#all!/0","title":"ToxiproxyEx.all!/0","type":"function"},{"doc":"Retrievs a proxy:\n    iex> ToxiproxyEx.create!(upstream: \"localhost:3306\", name: \"test_mysql_master\")\n    iex> ToxiproxyEx.create!(upstream: \"localhost:3307\", name: \"test_redis_master\")\n    iex> ToxiproxyEx.all!()","ref":"ToxiproxyEx.html#all!/0-examples","title":"Examples - ToxiproxyEx.all!/0","type":"function"},{"doc":"Applies all toxics previously defined on the list of proxies during the duration of the given function.\n\nRaises `ToxiproxyEx.ServerError` if the toxics could not be enabled and disabled again on the server.","ref":"ToxiproxyEx.html#apply!/2","title":"ToxiproxyEx.apply!/2","type":"function"},{"doc":"Add toxics and apply them toxic to a single proxy:\n    iex> proxy = ToxiproxyEx.create!(upstream: \"localhost:3306\", name: \"test_mysql_master\")\n    iex> proxies = ToxiproxyEx.downstream(proxy, :slow_close, delay: 100)\n    iex> proxies = ToxiproxyEx.downstream(proxies, :latency, jitter: 300)\n    iex> ToxiproxyEx.apply!(proxies, fn ->\n    ...>  # All calls to mysql master are now slow at responding and closing.\n    ...>  nil\n    ...> end)\n\nAdd toxics and apply them toxic to a list of proxies:\n    iex> ToxiproxyEx.create!(upstream: \"localhost:3306\", name: \"test_mysql_master\")\n    iex> ToxiproxyEx.create!(upstream: \"localhost:3306\", name: \"test_mysql_follower\")\n    iex> proxies = ToxiproxyEx.all!()\n    iex> proxies = ToxiproxyEx.downstream(proxies, :slow_close, delay: 100)\n    iex> proxies = ToxiproxyEx.downstream(proxies, :latency, jitter: 300)\n    iex> ToxiproxyEx.apply!(proxies, fn ->\n    ...>  # All calls to mysql master and follower are now slow at responding and closing.\n    ...>  nil\n    ...> end)","ref":"ToxiproxyEx.html#apply!/2-examples","title":"Examples - ToxiproxyEx.apply!/2","type":"function"},{"doc":"Creates a proxy on the toxiproxy server.\n\nRaises `ToxiproxyEx.ServerError` if the creation fails.","ref":"ToxiproxyEx.html#create!/1","title":"ToxiproxyEx.create!/1","type":"function"},{"doc":"Create a new proxy:\n    iex> ToxiproxyEx.create!(upstream: \"localhost:3306\", name: \"test_mysql_master\")\n\nCreate a new proxy that listens on a specific port:\n    iex> ToxiproxyEx.create!(upstream: \"localhost:3306\", listen: \"localhost:5555\", name: \"test_mysql_master\")\n\nCreate a new proxy that is disabled by default:\n    iex> ToxiproxyEx.create!(upstream: \"localhost:3306\", name: \"test_mysql_master\", enabled: false)","ref":"ToxiproxyEx.html#create!/1-examples","title":"Examples - ToxiproxyEx.create!/1","type":"function"},{"doc":"Deletes one or multiple proxies on the toxiproxy server.\n\nRaises `ToxiproxyEx.ServerError` if the deletion fails.","ref":"ToxiproxyEx.html#destroy!/1","title":"ToxiproxyEx.destroy!/1","type":"function"},{"doc":"Destroy a single proxy:\n    iex> ToxiproxyEx.create!(upstream: \"localhost:3456\", name: :test_mysql_master)\n    iex> proxy = ToxiproxyEx.get!(:test_mysql_master)\n    iex> ToxiproxyEx.destroy!(proxy)\n    :ok\n\nDestroy all proxies:\n    iex> ToxiproxyEx.create!(upstream: \"localhost:3456\", name: :test_mysql_master)\n    iex> proxies = ToxiproxyEx.all!()\n    iex> ToxiproxyEx.destroy!(proxies)\n    :ok","ref":"ToxiproxyEx.html#destroy!/1-examples","title":"Examples - ToxiproxyEx.destroy!/1","type":"function"},{"doc":"Takes down the proxy or the list of proxies during the duration of the given function.\n\nRaises `ToxiproxyEx.ServerError` if the proxy or list of proxies could not have been disabled and enabled again on the server.","ref":"ToxiproxyEx.html#down!/2","title":"ToxiproxyEx.down!/2","type":"function"},{"doc":"Take down a single proxy:\n    iex> proxy = ToxiproxyEx.create!(upstream: \"localhost:3306\", name: \"test_mysql_master\")\n    iex> ToxiproxyEx.down!(proxy, fn ->\n    ...>  # Takes mysql master down.\n    ...>  nil\n    ...> end)\n\nTake down a list of proxies:\n    iex> ToxiproxyEx.create!(upstream: \"localhost:3306\", name: \"test_mysql_master\")\n    iex> ToxiproxyEx.create!(upstream: \"localhost:3306\", name: \"test_mysql_follower\")\n    iex> proxies = ToxiproxyEx.all!()\n    iex> ToxiproxyEx.down!(proxies, fn ->\n    ...>  # Takes mysql master and follower down.\n    ...>  nil\n    ...> end)","ref":"ToxiproxyEx.html#down!/2-examples","title":"Examples - ToxiproxyEx.down!/2","type":"function"},{"doc":"Adds an downstream toxic to the proxy or list of proxies that will be enabled when passed to `ToxiproxyEx.apply!/2`.","ref":"ToxiproxyEx.html#downstream/3","title":"ToxiproxyEx.downstream/3","type":"function"},{"doc":"Add an downstream toxic to a proxy:\n    iex> proxy = ToxiproxyEx.create!(upstream: \"localhost:3306\", name: \"test_mysql_master\")\n    iex> proxies = ToxiproxyEx.downstream(proxy, :latency, latency: 1000)\n    iex> ToxiproxyEx.apply!(proxies, fn ->\n    ...>  # Do some testing\n    ...>  nil\n    ...> end)\n\nAdd an downstream toxic to a list of proxies:\n    iex> ToxiproxyEx.create!(upstream: \"localhost:3306\", name: \"test_mysql_master\")\n    iex> ToxiproxyEx.create!(upstream: \"localhost:3307\", name: \"test_mysql_follower\")\n    iex> proxies = ToxiproxyEx.all!()\n    iex> proxies = ToxiproxyEx.downstream(proxies, :latency, latency: 1000)\n    iex> ToxiproxyEx.apply!(proxies, fn ->\n    ...>  # Do some testing\n    ...>  nil\n    ...> end)","ref":"ToxiproxyEx.html#downstream/3-examples","title":"Examples - ToxiproxyEx.downstream/3","type":"function"},{"doc":"Retrieves a proxy from the toxiproxy server.\n\nRaises `ToxiproxyEx.ServerError` if the proxy could not be retrieved.\nRaises `ArgumentError` if the proxy does not exist.","ref":"ToxiproxyEx.html#get!/1","title":"ToxiproxyEx.get!/1","type":"function"},{"doc":"Retrievs a proxy:\n    iex> ToxiproxyEx.create!(upstream: \"localhost:3306\", name: \"test_mysql_master\")\n    iex> ToxiproxyEx.get!(:test_mysql_master)","ref":"ToxiproxyEx.html#get!/1-examples","title":"Examples - ToxiproxyEx.get!/1","type":"function"},{"doc":"Retrieves a list of proxies from the toxiproxy server where the name matches the specificed regex.\n\nRaises `ToxiproxyEx.ServerError` if the list of proxies could not be retrieved.\nRaises `ArgumentError` if no proxy matching the specified regex does exist.","ref":"ToxiproxyEx.html#grep!/1","title":"ToxiproxyEx.grep!/1","type":"function"},{"doc":"Retrievs a proxy:\n    iex> ToxiproxyEx.create!(upstream: \"localhost:3306\", name: \"test_mysql_master\")\n    iex> ToxiproxyEx.create!(upstream: \"localhost:3307\", name: \"test_mysql_follower\")\n    iex> ToxiproxyEx.create!(upstream: \"localhost:3308\", name: \"test_redis_master\")\n    iex> ToxiproxyEx.grep!(~r/master/)","ref":"ToxiproxyEx.html#grep!/1-examples","title":"Examples - ToxiproxyEx.grep!/1","type":"function"},{"doc":"Creates proxies based on the passed data.\nThis is usefull to quickly create multiple proxies based on hardcoded value or values read from external sources such as a config file.\n\nNonexisting proxies will be created and existing ones will be updated to match the passed data.\n\nRaises `ToxiproxyEx.ServerError` if the proxies could not have been created on the server.","ref":"ToxiproxyEx.html#populate!/1","title":"ToxiproxyEx.populate!/1","type":"function"},{"doc":"Creating proxies:\n    iex> ToxiproxyEx.populate!([\n    ...>  %{name: \"test_mysql_master\", upstream: \"localhost:5765\"},\n    ...>  %{name: \"test_mysql_follower\", upstream: \"localhost:5766\", enabled: false}\n    ...> ])","ref":"ToxiproxyEx.html#populate!/1-examples","title":"Examples - ToxiproxyEx.populate!/1","type":"function"},{"doc":"Re-enables are proxies and disables all toxics on toxiproxy.\n\nRaises `ToxiproxyEx.ServerError` if the server could not have been reset.","ref":"ToxiproxyEx.html#reset!/0","title":"ToxiproxyEx.reset!/0","type":"function"},{"doc":"Reset toxiproxy:\n    iex> ToxiproxyEx.reset!()\n    :ok","ref":"ToxiproxyEx.html#reset!/0-examples","title":"Examples - ToxiproxyEx.reset!/0","type":"function"},{"doc":"Alias for `ToxiproxyEx.downstream/3`.","ref":"ToxiproxyEx.html#toxic/3","title":"ToxiproxyEx.toxic/3","type":"function"},{"doc":"Alias for `ToxiproxyEx.downstream/3`.","ref":"ToxiproxyEx.html#toxicate/3","title":"ToxiproxyEx.toxicate/3","type":"function"},{"doc":"Adds an upstream toxic to the proxy or list of proxies that will be enabled when passed to `ToxiproxyEx.apply!/2`.","ref":"ToxiproxyEx.html#upstream/3","title":"ToxiproxyEx.upstream/3","type":"function"},{"doc":"Add an upstream toxic to a proxy:\n    iex> proxy = ToxiproxyEx.create!(upstream: \"localhost:3306\", name: \"test_mysql_master\")\n    iex> proxies = ToxiproxyEx.upstream(proxy, :latency, latency: 1000)\n    iex> ToxiproxyEx.apply!(proxies, fn ->\n    ...>  # Do some testing\n    ...>  nil\n    ...> end)\n\nAdd an upstream toxic to a list of proxies:\n    iex> ToxiproxyEx.create!(upstream: \"localhost:3306\", name: \"test_mysql_master\")\n    iex> ToxiproxyEx.create!(upstream: \"localhost:3307\", name: \"test_mysql_follower\")\n    iex> proxies = ToxiproxyEx.all!()\n    iex> proxies = ToxiproxyEx.upstream(proxies, :latency, latency: 1000)\n    iex> ToxiproxyEx.apply!(proxies, fn ->\n    ...>  # Do some testing\n    ...>  nil\n    ...> end)","ref":"ToxiproxyEx.html#upstream/3-examples","title":"Examples - ToxiproxyEx.upstream/3","type":"function"},{"doc":"Gets the version of the running toxiproxy server.\n\nRaises `ToxiproxyEx.ServerError` if the version could not have been fetched from the server.","ref":"ToxiproxyEx.html#version!/0","title":"ToxiproxyEx.version!/0","type":"function"},{"doc":"Get running toxiproxy version:\n    iex> ToxiproxyEx.version!()\n    \"2.1.2\"","ref":"ToxiproxyEx.html#version!/0-examples","title":"Examples - ToxiproxyEx.version!/0","type":"function"},{"doc":"A hostname or IP address including a port number, e.g. `localhost:4539`.","ref":"ToxiproxyEx.html#t:host_with_port/0","title":"ToxiproxyEx.host_with_port/0","type":"type"},{"doc":"A proxy that intercepts traffic to and from an upstream server.","ref":"ToxiproxyEx.html#t:proxy/0","title":"ToxiproxyEx.proxy/0","type":"opaque"},{"doc":"A map containing fields required to setup a proxy. Designed to be used with `ToxiproxyEx.populate!/1`.","ref":"ToxiproxyEx.html#t:proxy_map/0","title":"ToxiproxyEx.proxy_map/0","type":"type"},{"doc":"A collection of proxies.","ref":"ToxiproxyEx.html#t:toxic_collection/0","title":"ToxiproxyEx.toxic_collection/0","type":"opaque"},{"doc":"Raised when communication with the toxiproxy server fails.","ref":"ToxiproxyEx.ServerError.html","title":"ToxiproxyEx.ServerError","type":"exception"},{"doc":"# ToxiproxyEx\n\n[![github.com](https://github.com/Jcambass/toxiproxy_ex/workflows/.github/workflows/main.yml/badge.svg)](https://github.com/Jcambass/toxiproxy_ex/actions)\n[![hex.pm](https://img.shields.io/hexpm/v/toxiproxy_ex.svg)](https://hex.pm/packages/toxiproxy_ex)\n[![hexdocs.pm](https://img.shields.io/badge/api-docs-lightgreen.svg)](https://hexdocs.pm/toxiproxy_ex/api-reference.html)\n[![hex.pm](https://img.shields.io/hexpm/dt/toxiproxy_ex.svg)](https://hex.pm/packages/toxiproxy_ex)\n[![hex.pm](https://img.shields.io/hexpm/l/toxiproxy_ex.svg)](https://hex.pm/packages/toxiproxy_ex)\n[![github.com](https://img.shields.io/github/last-commit/Jcambass/toxiproxy_ex.svg)](https://github.com/Jcambass/toxiproxy_ex/commits/main)\n\n \n\nToxiproxyEx is an Elixir API client for the resilience testing tool Toxiproxy.\n\n[Toxiproxy](https://github.com/shopify/toxiproxy) is a proxy to simulate network\nand system conditions. The Elixir API aims to make it simple to write tests that\nensure your application behaves appropriately under harsh conditions. Before you\ncan use the Elixir library, you need to read the [Usage section of the Toxiproxy\nREADME](https://github.com/shopify/toxiproxy#usage).","ref":"readme.html","title":"ToxiproxyEx","type":"extras"},{"doc":"By default the Elixir client communicates with the Toxiproxy daemon via HTTP on `http://127.0.0.1:8474`, but you can point to any host via your application configuration:\n```elixir\nconfig :toxiproxy_ex, host: \"http://toxiproxy.local:8844\"\n```\n\nFor example, to simulate 1000ms latency on a database server you can use the\n`latency` toxic with the `latency` argument (see the Toxiproxy project for a\nlist of all toxics):\n\n```elixir\nToxiproxyEx.get!(:mysql_master)\n|> ToxiproxyEx.toxic(:latency, latency: 1000)\n|> ToxiproxyEx.apply!(fn ->\n  Repo.all(Shop) # this took at least 1s\nend)\n```\n\nYou can also take an endpoint down for the duration of a function at the TCP level:\n\n```elixir\nToxiproxyEx.get!(:mysql_master)\n|> ToxiproxyEx.down!(fn ->\n  Repo.all(Shop) # this'll raise\nend)\n```\n\nIf you want to simulate all your Redis instances being down:\n\n```elixir\nToxiproxyEx.grep!(~r/redis/)\n|> ToxiproxyEx.down!(fn ->\n  # any redis call will fail\nend)\n```\n\nIf you want to simulate that your cache server is slow at incoming network\n(upstream), but fast at outgoing (downstream), you can apply a toxic to just the\nupstream:\n\n```elixir\nToxiproxyEx.get!(:cache)\n|> ToxiproxyEx.upstream(:latency, latency: 1000)\n|> ToxiproxyEx.apply!(fn ->\n  Cache.get(:omg) # will take at least a second\nend)\n```\n\nBy default the toxic is applied to the downstream connection, you can be\nexplicit and compose them:\n\n```elixir\nToxiproxyEx.grep!(~r/redis/)\n|> ToxiproxyEx.upstream(:slow_close, delay: 100)\n|> ToxiproxyEx.downstream(:latency, jitter: 300)\n|> ToxiproxyEx.apply!(fn ->\n  # all redises are now slow at responding and closing\nend)\n```\n\nSee the [Toxiproxy README](https://github.com/shopify/toxiproxy#Toxics) for a\nlist of toxics.","ref":"readme.html#usage","title":"Usage - ToxiproxyEx","type":"extras"},{"doc":"To populate Toxiproxy pass the proxy configurations to `ToxiproxyEx.populate!`:\n\n```elixir\nToxiproxyEx.populate!([\n  %{\n    name: \"mysql_master\",\n    listen: \"localhost:21212\",\n    upstream: \"localhost:3306\",\n  },\n  %{\n    name: \"mysql_read_only\",\n    listen: \"localhost:21213\",\n    upstream: \"localhost:3306\",\n  }\n])\n```\n\nThis will create the proxies passed, or replace the proxies if they already exist in Toxiproxy.\nIt's recommended to do this as early in your application startup process as possible, see the\n[Toxiproxy README](https://github.com/shopify/toxiproxy#usage). If you have many\nproxies, we recommend storing the Toxiproxy configs in a configuration file and\ndeserializing it into `ToxiproxyEx.populate!/1`.","ref":"readme.html#populate","title":"Populate - ToxiproxyEx","type":"extras"},{"doc":"This library made the choice to use exceptions on the public API methods to signal errors.\n\nThis was chosen since this is a library meant to be used in testing code only, where you want test cases to fail if your set assumptions are not met. In this sense setting assumptions that will not be met (toxiproxy-server is not running, passing invalid configurations) is considered to be a developer error and should be fixed rather than handled in code.\n\n**Server Errors**\n\nIf any API interaction with toxiproxy fails, a `ServerError` will be raised.\n\n**Client Errors**\n\nIf you miss-configure toxiproxy via the elixir API, an `ArgumentError` will be raised.","ref":"readme.html#error-handling","title":"Error Handling - ToxiproxyEx","type":"extras"},{"doc":"The package can be installed\nby adding `toxiproxy_ex` to your list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:toxiproxy_ex, \"~> 1.1.1\", only: :test}\n  ]\nend\n```","ref":"readme.html#installation","title":"Installation - ToxiproxyEx","type":"extras"},{"doc":"Clone the repo and fetch its dependencies:\n\n    $ git clone https://github.com/jcambass/toxiproxy_ex.git\n    $ cd toxiproxy_ex\n    $ mix deps.get\n\nMake sure that you have [Toxiproxy](https://github.com/Shopify/toxiproxy) installed and start it:\n\n    $ toxiproxy-server\n\nRun tests:\n\n    $ mix test","ref":"readme.html#running-tests","title":"Running tests - ToxiproxyEx","type":"extras"}]}